#include "Application.hpp"
#include "quark.hpp"
#include <spdlog/spdlog.h>

namespace qk {

	Application::Application() {
		window = Window::Create();
		window->SetEventCallback(std::bind(&Application::OnEvent, this, std::placeholders::_1));
	}

	void Application::Run() {
		while(isRunning) {
			for (Layer* layer : layers) {
				layer->OnUpdate();
			}

			window->OnUpdate();
		}
	}

	void Application::OnEvent(Event& e) {
		QK_CORE_TRACE(e);

		EventDispatcher dispatcher(e);
		dispatcher.Dispatch<WindowCloseEvent>(std::bind(&Application::OnWindowClose, this, std::placeholders::_1));

		for (auto it = layers.rbegin(); it != layers.rend(); it++) {
			(*it)->OnEvent(e);
			if (e.isHandled) {
				break;
			}
		}
	}

	void Application::PushLayer(Layer* layer) {
		layers.PushLayer(layer);
	}

	void Application::PushOverlay(Layer* layer) {
		layers.PushOverlay(layer);
	}

	bool Application::OnWindowClose(WindowCloseEvent& e) {
		isRunning = false;
		return true;
	}
}
